import json
from base64 import b64decode, b64encode
from typing import Dict
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import padding
from cryptography.hazmat.primitives import hashes
from Osdental.Exception.ControlledException import RSAEncryptException
from Osdental.Shared.Logger import logger
from Osdental.Shared.Enums.Message import Message
from Osdental.Shared.Enums.Constant import Constant

class RSAEncryptor:

    @staticmethod
    def encrypt(data:str | Dict[str,str], public_key_rsa:str) -> str:
        try:
            if isinstance(data, dict):
                data = json.dumps(data)

            public_key = serialization.load_pem_public_key(public_key_rsa.encode(Constant.DEFAULT_ENCODING))
            data_bytes = data.encode(Constant.DEFAULT_ENCODING)
            encrypted_bytes = public_key.encrypt(
                data_bytes,
                padding.OAEP(
                    mgf=padding.MGF1(algorithm=hashes.SHA256()),
                    algorithm=hashes.SHA256(),
                    label=None
                )
            )

            return b64encode(encrypted_bytes).decode(Constant.DEFAULT_ENCODING)

        except Exception as e:
            logger.error(f'Unexpected RSA encryption error: {str(e)}')
            raise RSAEncryptException(message=Message.UNEXPECTED_ERROR_MSG, error=str(e)) 


    @staticmethod
    def decrypt(data:str, private_key_rsa:str, silent:bool = False) -> str:
        try:
            encrypted_bytes = b64decode(data)
            private_key = serialization.load_pem_private_key(private_key_rsa.encode(Constant.DEFAULT_ENCODING), password=None)
            decrypted_bytes = private_key.decrypt(
                encrypted_bytes,
                padding.OAEP(
                    mgf=padding.MGF1(algorithm=hashes.SHA256()),
                    algorithm=hashes.SHA256(),
                    label=None
                )
            )

            return decrypted_bytes.decode(Constant.DEFAULT_ENCODING)

        except Exception as e:
            if not silent:
                logger.error(f'Unexpected RSA decryption error: {str(e)}')
                
            raise RSAEncryptException(message=Message.UNEXPECTED_ERROR_MSG, error=str(e))


import inspect
import json
from functools import wraps
from typing import Callable, Tuple, Optional, Union, Dict, Any
from uuid import UUID
from graphql import GraphQLResolveInfo
from graphql import OperationType
from Osdental.Encryptor.Jwt import JWT
from Osdental.Exception.ControlledException import UnauthorizedException, ProfilePermissionDeniedException, InvalidFormatException
from Osdental.Handlers.DBSecurityQuery import DBSecurityQuery
from Osdental.Models.Legacy import Legacy
from Osdental.Handlers.Instances import jwt_user_key, aes
from Osdental.Models.Token import AuthToken
from Osdental.Shared.Enums.Message import Message
from Osdental.Shared.Enums.Profile import Profile

db_security_query = DBSecurityQuery()

async def validate_token(token_id: str, user_id: str):
    is_auth = await db_security_query.validate_auth_token(token_id, user_id)
    if not is_auth:
        raise UnauthorizedException(message=Message.PORTAL_ACCESS_RESTRICTED_MSG, error=Message.PORTAL_ACCESS_RESTRICTED_MSG)
    

def check_profile_permission(allowed_permissions: str | Tuple[str, ...] | None, requested_permission: str) -> bool:
    SUPER_PROFILES = (Profile.SUPER_ADMIN, Profile.ADMIN_OSD)

    if allowed_permissions is None:
        allowed = ()
    elif isinstance(allowed_permissions, str):
        allowed = (allowed_permissions,)
    else:
        allowed = allowed_permissions

    total_allowed = set(allowed) | set(SUPER_PROFILES)
    return requested_permission in total_allowed


async def decrypted_token(info: GraphQLResolveInfo = None, legacy: Legacy = None, mutate: bool = True) -> Optional[AuthToken]:
    operation_type = info.operation.operation
    user_token_encrypted = info.context.get('user_token')

    if not user_token_encrypted:
        return None

    user_token = aes.decrypt(legacy.aes_key_user, user_token_encrypted)
    payload = JWT.extract_payload(user_token, jwt_user_key)
    payload['legacy'] = legacy
    payload['jwt_user_key'] = jwt_user_key
    token = AuthToken.from_jwt(payload)
    token.base_id_external_enterprise = token.id_external_enterprise
    await validate_token(token.id_token, token.id_user)

    headers = info.context.get('headers', {})
    id_external_mk = headers.get('dynamicClientId')

    is_marketing = token.abbreviation.startswith(Profile.MARKETING)
    should_use_zero_uuid = (
        token.abbreviation.startswith((Profile.SUPER_ADMIN, Profile.ADMIN_OSD))
        and operation_type == OperationType.QUERY and mutate
    )
    should_use_mk_header = is_marketing and id_external_mk

    if should_use_zero_uuid:
        token.id_external_enterprise = str(UUID(int=0))
    elif should_use_mk_header:
        token.id_external_enterprise = aes.decrypt(token.aes_key_auth, id_external_mk)
        token.mk_id_external_enterprise = id_external_mk

    return token


def decrypted_data(aes_data: Optional[str], aes_key_auth: str, token: AuthToken) -> Optional[Dict[str, Any]]:
    data = None
    if aes_data is not None:
        decrypted_data = aes.decrypt(aes_key_auth, aes_data)
        if isinstance(decrypted_data, str):
            try:
                data = json.loads(decrypted_data)
            except Exception:
                raise InvalidFormatException(message=Message.INVALID_AES_JSON_FORMAT_MSG)
        elif isinstance(decrypted_data, dict):
            data = decrypted_data
        else:
            raise UnauthorizedException(message=Message.UNEXPECTED_DECRYPTED_DATA_FORMAT_MSG)
        # Overwrite enterprise ID if applicable
        external_enterprise_req = data.get('idExternalEnterprise')
        if external_enterprise_req and token:
            token.id_external_enterprise = external_enterprise_req

    return data

def process_encrypted_data(mutate: bool = True, allowed_permissions: Optional[Union[str, Tuple[str, ...]]] = None):
    def decorator(func: Callable):
        @wraps(func)
        async def wrapper(self, info: GraphQLResolveInfo = None, aes_data: str = None, **rest_kwargs):
            legacy = await db_security_query.get_legacy_data()
            token = await decrypted_token(info, legacy, mutate) if info else None

            if allowed_permissions and token:
                is_authorized = check_profile_permission(allowed_permissions, token.abbreviation)
                if not is_authorized:
                    raise ProfilePermissionDeniedException(message=Message.PROFILE_PERMISSION_DENIED_MSG)

            data = decrypted_data(aes_data, legacy.aes_key_auth, token)
            
            # Introspect function params
            sig = inspect.signature(func)
            kwargs_to_pass = {}
            if 'token' in sig.parameters and token:
                kwargs_to_pass['token'] = token
            if 'data' in sig.parameters and data:
                kwargs_to_pass['data'] = data

            return await func(self, **kwargs_to_pass, **rest_kwargs)

        return wrapper
    return decorator


from __future__ import annotations
from dataclasses import dataclass
from typing import Dict
from Osdental.Shared.Utils.CaseConverter import CaseConverter
from Osdental.Exception.ControlledException import MissingFieldException
from Osdental.Shared.Enums.Message import Message

@dataclass
class Legacy:
    id_legacy: str
    legacy_name: str
    id_enterprise: str
    refresh_token_exp_min: int
    access_token_exp_min: int
    public_key2: str
    private_key1: str
    private_key2: str
    aes_key_user: str
    aes_key_auth: str

    def __post_init__(self):
        if not self.legacy_name:
            raise MissingFieldException(message=Message.LEGACY_NAME_REQUIRED_MSG)
        
        if not self.refresh_token_exp_min:
            raise MissingFieldException(message=Message.REFRESH_TOKEN_EXP_REQUIRED_MSG)
        
        if not self.access_token_exp_min:
            raise MissingFieldException(message=Message.ACCESS_TOKEN_EXP_REQUIRED_MSG)
        
        if not self.public_key2:
            raise MissingFieldException(message=Message.PUBLIC_KEY2_REQUIRED_MSG)
        
        if not self.private_key1:
            raise MissingFieldException(message=Message.PRIVATE_KEY1_REQUIRED_MSG)
        
        if not self.private_key2:
            raise MissingFieldException(message=Message.PRIVATE_KEY2_REQUIRED_MSG)
        
        if not self.aes_key_user:
            raise MissingFieldException(message=Message.AES_KEY_USER_REQUIRED_MSG)
        
        if not self.aes_key_auth:
            raise MissingFieldException(message=Message.AES_KEY_AUTH_REQUIRED_MSG)
    
    @classmethod
    def from_db(cls, record: Dict[str,str]) -> Legacy:
        mapped = {CaseConverter.case_to_snake(key) : value for key, value in record.items()}
        valid_fields = cls.__dataclass_fields__.keys()
        clean = {k: v for k, v in mapped.items() if k in valid_fields}
        return cls(**clean)




import os
import json
import base64
from typing import Dict, List
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from Osdental.Exception.ControlledException import AESEncryptException
from Osdental.Shared.Logger import logger
from Osdental.Shared.Enums.Message import Message
from Osdental.Shared.Enums.Constant import Constant

class AES:

    def __init__(self):
        self.IV_LENGTH = 32
        self.TAG_LENGTH = 16

    @staticmethod
    def generate_key() -> str:
        """Generates a random 256-bit AES key and returns it Base64 encoded."""
        key = os.urandom(32)
        return base64.b64encode(key).decode(Constant.DEFAULT_ENCODING)
    
    def encrypt(self, aes_key:str, data:Dict[str,str] | str | List[Dict[str,str]]) -> str:
        """
        Encrypts data using AES-GCM.
        Supports dictionary, string, or list inputs.

        :param aes_key: AES key in Base64 format.
        :param data: Data to be encrypted (dict, str, or list).
        :return: Data encrypted in Base64.
        """
        if not isinstance(data, (dict, str, list)):
            raise ValueError(AES_INPUT)

        try:
            key = base64.b64decode(aes_key)
            iv = os.urandom(self.IV_LENGTH)
            if isinstance(data, (dict, list)):
                json_data = json.dumps(data)
            else:
                json_data = data

            cipher = Cipher(algorithms.AES(key), modes.GCM(iv))
            encryptor = cipher.encryptor()
            ciphertext = encryptor.update(json_data.encode(Constant.DEFAULT_ENCODING)) + encryptor.finalize()
            tag = encryptor.tag
            encrypted_data = iv + ciphertext + tag

            return base64.b64encode(encrypted_data).decode(Constant.DEFAULT_ENCODING)

        except Exception as e:
            logger.error(f'Unexpected AES encryption error: {str(e)}')
            raise AESEncryptException(message=Message.UNEXPECTED_ERROR_MSG, error=str(e))
 

    def decrypt(self, aes_key:str, encrypted_data:str, silent:bool = False):
        """
        Decrypts data using AES-GCM.
        Expects encrypted data to represent either a JSON object (dict) or a plain string.

        :param aes_key: AES key in Base64 format.
        :param encrypted_data: Data encrypted in Base64.
        :return: Decrypted data (either dict or str).
        """
        try:
            key = base64.b64decode(aes_key)
            encrypted_data = base64.b64decode(encrypted_data)
            iv = encrypted_data[:self.IV_LENGTH]
            tag = encrypted_data[-self.TAG_LENGTH:]
            ciphertext = encrypted_data[self.IV_LENGTH:-self.TAG_LENGTH]
            cipher = Cipher(algorithms.AES(key), modes.GCM(iv, tag))
            decryptor = cipher.decryptor()
            plaintext = decryptor.update(ciphertext) + decryptor.finalize()

            try:
                decrypted_data = json.loads(plaintext.decode(Constant.DEFAULT_ENCODING))
                return decrypted_data
            except json.JSONDecodeError:
                return plaintext.decode(Constant.DEFAULT_ENCODING)

        except Exception as e:
            if not silent:
                logger.error(f'Unexpected AES decryption error: {str(e)}')
                
            raise AESEncryptException(message=Message.UNEXPECTED_ERROR_MSG, error=str(e))


import json
from typing import Dict, List, Optional
import redis.asyncio as redis
from Osdental.Exception.ControlledException import RedisException
from Osdental.Shared.Logger import logger
from Osdental.Shared.Enums.Message import Message
from Osdental.Shared.Enums.Constant import Constant

class RedisCacheAsync:

    _instances: Dict[str, 'RedisCacheAsync'] = {}

    def __new__(cls, redis_url:str):
        if redis_url not in cls._instances:
            cls._instances[redis_url] = super(RedisCacheAsync, cls).__new__(cls)
        return cls._instances[redis_url]

    def __init__(self, redis_url:str):
        """Connect to the Redis server."""
        if not hasattr(self, 'initialized'):
            self.pool = redis.ConnectionPool.from_url(redis_url)
            self.client = redis.Redis(connection_pool=self.pool)
            self.initialized = True


    async def set_dict(self, key:str, value:Dict[str,str], ttl:Optional[int] = None):
        """Set a JSON value in the cache."""
        try:
            json_value = json.dumps(value)
            await self.client.set(key, json_value)
            if ttl:
                await self.client.expire(key, ttl)
        except Exception as e:
            logger.error(f'Redis set_dict error: {str(e)}')
            raise RedisException(message=Message.UNEXPECTED_ERROR_MSG, error=str(e))

    async def set_str(self, key:str, value:str, ttl:Optional[int] = None):
        """Set a string value in the cache."""
        try:
            await self.client.set(key, value)
            if ttl:
                await self.client.expire(key, ttl)
        except Exception as e:
            logger.error(f'Redis set_str error: {str(e)}')
            raise RedisException(message=Message.UNEXPECTED_ERROR_MSG, error=str(e))

    async def get_dict(self, key:str) -> Optional[str]:
        """Get a JSON value from the cache and convert it back to a Python object."""
        try:
            json_value = await self.client.get(key)
            if json_value:
                return json.loads(json_value)
            return None
        except Exception as e:
            logger.error(f'Redis get_dict error: {str(e)}')
            raise RedisException(message=Message.UNEXPECTED_ERROR_MSG, error=str(e))


    async def get_str(self, key:str) -> Optional[str]:
        """Get a string value from the cache."""
        try:
            value = await self.client.get(key)
            return value.decode(Constant.DEFAULT_ENCODING) if value else None
        except Exception as e:
            logger.error(f'Redis get_str error: {str(e)}')
            raise RedisException(message=Message.UNEXPECTED_ERROR_MSG, error=str(e))

    async def delete(self, key:str) -> bool:
        """Delete a value from the cache."""
        try:
            return await self.client.delete(key)
        except Exception as e:
            logger.error(f'Redis delete error: {str(e)}')
            raise RedisException(message=Message.UNEXPECTED_ERROR_MSG, error=str(e))

    async def exists(self, key:str) -> bool:
        """Check if a key exists in the cache."""
        return await self.client.exists(key)

    async def flush(self):
        """Flush all keys in the cache."""
        await self.client.flushdb()
    
    async def flush_all(self):
        """Flush all keys in all Redis databases."""
        await self.client.flushall()

    async def mget(self, keys:List[str]) -> List[Optional[str]]:
        """Get multiple values from the cache."""
        values = await self.client.mget(keys)
        return [json.loads(value) if value else None for value in values]

    async def clear_cache(self, prefix: str):
        """Delete all keys matching the given prefix."""
        async for key in self._scan_keys(match=f'{prefix}*'):
            await self.client.delete(key)

    async def _scan_keys(self, match: str = '*'):
        """Asynchronous generator to scan keys matching a pattern."""
        async for key in self.client.scan_iter(match=match):
            yield key

    async def close(self):
        """Close the connection pool and Redis client."""
        if self.client:
            await self.client.close()  
        if self.pool:
            await self.pool.disconnect(inuse_connections=True) 



import os
import json
import asyncio
import httpx
from datetime import datetime
from typing import Dict
from dotenv import load_dotenv
from tenacity import retry, stop_after_attempt, wait_exponential, retry_if_exception_type
from Osdental.ServicesBus.TaskQueue import task_queue
from Osdental.Exception.ControlledException import HttpClientException
from Osdental.Shared.Logger import logger
from Osdental.Shared.Enums.Message import Message
from Osdental.Shared.Enums.Constant import Constant

load_dotenv(dotenv_path='.env', override=True)

class APIClient(httpx.AsyncClient):

    def __init__(self, **kwargs):
        timeout = kwargs.pop('timeout', httpx.Timeout(10.0, read=20.0))
        super().__init__(follow_redirects=True, timeout=timeout, **kwargs)

    @retry(
        retry=retry_if_exception_type(httpx.RequestError),
        stop=stop_after_attempt(3),  
        wait=wait_exponential(multiplier=1, min=2, max=10),  
        reraise=True
    )
    async def _perform_request(self, method: str, url: str, *args, **kwargs):
        response = await self.request(method, url, *args, **kwargs)
        response.raise_for_status()
        return response
    
    async def rest_request(self, method:str, url:str, *args, **kwargs) -> str:
        try:
            headers = kwargs.get('headers', {})
            asyncio.create_task(self.send_request_to_service_bus(endpoint=url, body=kwargs.get('body'), headers=headers))
            response = await self._perform_request(method, url, *args, **kwargs)    
            asyncio.create_task(self.send_response_to_service_bus(response, headers=headers))
            content_type = response.headers.get('content-type', '').lower()
            if 'application/json' in content_type:
                return response.json()
            elif 'text/' in content_type or 'html' in content_type or 'xml' in content_type:
                return response.text
            else:
                return response.content
        
        except httpx.RequestError as e:
            logger.error(f'Unexpected rest request error: {str(e)}')
            raise HttpClientException(message=Message.UNEXPECTED_ERROR_MSG, error=str(e))


    @retry(
        retry=retry_if_exception_type(httpx.RequestError),
        stop=stop_after_attempt(3),
        wait=wait_exponential(multiplier=1, min=2, max=10),
        reraise=True
    )
    async def _perform_graphql_request(self, url: str, consult: str, variables: Dict[str, str], headers: dict):
        response = await self.post(url, json={'query': consult, 'variables': variables}, headers=headers)
        response.raise_for_status()
        return response
    
    async def graphql_request(self, url:str, consult:str, variables:Dict[str,str], **kwargs) -> str:
        try:
            headers = kwargs.get('headers', {})
            asyncio.create_task(self.send_request_to_service_bus(endpoint=url, body=variables, headers=headers))
            response = await self._perform_graphql_request(url, consult, variables, headers)
            asyncio.create_task(self.send_response_to_service_bus(response, headers=headers))
            content_type = response.headers.get('content-type', '').lower()
            if 'application/json' in content_type:
                return response.json()
            elif 'text/' in content_type or 'html' in content_type or 'xml' in content_type:
                return response.text
            else:
                return response.content
        
        except httpx.RequestError as e:
            logger.error(f'Unexpected grahql request error: {str(e)}')
            raise HttpClientException(message=Message.UNEXPECTED_ERROR_MSG, error=str(e))
        

    async def send_request_to_service_bus(self, endpoint:str, body:Dict[str,str], headers:Dict[str,str]) -> None:
        """
        Send a message to the Service Bus with details about the request made:

        :param endpoint: (str): URL of the endpoint to which the request will be made.
        :param body: (Dict[str, Any]): Body of the request (usually in JSON format).
        """
        message_json = {
            'idMessageLog': headers.get('Idmessagelog'),
            'type': 'REQUEST',
            'environment': os.getenv('ENVIRONMENT'),
            'dateExecution': datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
            'header': json.dumps(dict(headers)),
            'microServiceUrl': endpoint,
            'microServiceName': os.getenv('MICROSERVICE_NAME'),
            'microServiceVersion': os.getenv('MICROSERVICE_VERSION'),
            'serviceName': body.get('operationName') if body else '*',
            'machineNameUser': headers.get('Machinenameuser'),
            'ipUser': headers.get('Ipuser'),
            'userName': headers.get('Username'),
            'localitation': headers.get('Localitation'),
            'httpMethod': 'POST',
            'httpResponseCode': '*',
            'messageIn': json.dumps(body) if body else '*',
            'messageOut': '*',
            'errorProducer': '*',
            'auditLog': Constant.MESSAGE_LOG_EXTERNAL
        }
        await task_queue.enqueue(message_json)

    async def send_response_to_service_bus(self, response:httpx.Response, headers:Dict[str,str]) -> None:
        """
        Send a message to the Service Bus with details about the response received:

        :param response: (Response): Response object received from the endpoint.
        """
        message_json = {
            'idMessageLog': headers.get('Idmessagelog'),
            'type': 'RESPONSE',
            'dateExecution': datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
            'httpResponseCode': str(response.status_code),
            'messageOut': json.dumps(response.json()),
            'auditLog': Constant.MESSAGE_LOG_EXTERNAL
        }
        await task_queue.enqueue(message_json)


import uuid
from typing import Dict

class DataUtils:

    @staticmethod
    def normalize_uuid_value(value):
        if isinstance(value, str):
            try:
                return str(uuid.UUID(value)).lower()
            except ValueError:
                return value
        return value

    @staticmethod
    def normalize_uuids_dict(data: Dict[str,str]) -> Dict[str,str]:
        return {k: DataUtils.normalize_uuid_value(v) for k, v in data.items()}


import json
import asyncio
from typing import List, Dict, Any
from functools import wraps
from Osdental.InternalHttp.Request import CustomRequest
from Osdental.InternalHttp.Response import CustomResponse
from Osdental.Encryptor.Rsa import RSAEncryptor
from Osdental.Exception.ControlledException import OSDException, RSAEncryptException, AESEncryptException
from Osdental.Handlers.DBSecurityQuery import DBSecurityQuery
from Osdental.Handlers.Instances import aes
from Osdental.Shared.Utils.TextProcessor import TextProcessor
from Osdental.Shared.Logger import logger

def split_into_batches(data: List[Any], batch:int = 250):
    for i in range(0, len(data), batch):
        yield data[i:i + batch]

def try_decrypt_or_return_raw(data: str, private_key_rsa: str, aes_key: str) -> str:
    try:
        return RSAEncryptor.decrypt(data, private_key_rsa, silent=True)
    except RSAEncryptException:
        try:
            return aes.decrypt(aes_key, data, silent=True)
        except AESEncryptException:
            return data

def enqueue_response(data: Any, batch: int, headers: Dict[str,str], msg_info: str = None):
    if data and isinstance(data, list):
        if batch > 0 and len(data) > batch:
            batches = split_into_batches(data, batch)
            for idx, data_batch in enumerate(batches, start=1):
                custom_response = CustomResponse(content=json.dumps(data_batch), headers=headers, batch=idx)
                _ = asyncio.create_task(custom_response.send_to_service_bus())
        else:
            custom_response = CustomResponse(content=json.dumps(data), headers=headers)
            _ = asyncio.create_task(custom_response.send_to_service_bus())
    else:
        content = json.dumps(data) if isinstance(data, dict) else msg_info
        custom_response = CustomResponse(content=content, headers=headers)
        _ = asyncio.create_task(custom_response.send_to_service_bus())

def handle_audit_and_exception(batch: int = 0):
    def decorator(func):
        @wraps(func)
        async def wrapper(*args, **kwargs):
            try:
                headers = {}
                legacy = await DBSecurityQuery.get_legacy_data()
                _, info = args[:2] 
                request = info.context.get('request')
                headers = info.context.get('headers') or {}
                if request:
                    custom_request = CustomRequest(request)
                    await custom_request.send_to_service_bus()

                response = await func(*args, **kwargs)
                msg_info = TextProcessor.concatenate(response.get('status'), '-', response.get('message'))
                raw_data = response.get('data')
                data = None
                if raw_data:
                    data = try_decrypt_or_return_raw(raw_data, legacy.private_key2, legacy.aes_key_auth)

                enqueue_response(data, batch, headers, msg_info)
                return response

            except OSDException as ex:
                logger.warning(f'Controlled server error: {str(ex.error)}')
                ex.headers = headers
                _ = asyncio.create_task(ex.send_to_service_bus())
                return ex.get_response()
            
            except Exception as e:
                logger.error(f'Unexpected server error: {str(e)}')            
                ex = OSDException(error=str(e), headers=headers)
                _ = asyncio.create_task(ex.send_to_service_bus())
                return ex.get_response()

        return wrapper
    return decorator


from Osdental.Handlers.Instances import db_connection
from Osdental.Models.CDataIntegration import CdataIntegration, CdataInfo

class DBConnectionQuery:

    @staticmethod
    async def get_cdata_integration_data() -> CdataIntegration:
        data = await db_connection.execute_query_return_data('EXEC CONNECT.sps_GetCDataCredentials', fetchone=True)
        return CdataIntegration.from_db(data)

    @staticmethod
    async def get_cdata_integration_catalog_data() -> CdataInfo:
        data = await db_connection.execute_query_return_data('EXEC CATALOG.sps_GetCDataInfoFromCatalog')
        catalog = {}
        for el in data:
            catalog[el.get('name')] = el.get('value') 
        return CdataInfo.from_db(catalog)



from Osdental.Handlers.Instances import db_security
from Osdental.Models.Legacy import Legacy
from Osdental.Exception.ControlledException import UnauthorizedException
from Osdental.Shared.Enums.Message import Message
from Osdental.Shared.Enums.Code import Code

class DBSecurityQuery:

    @staticmethod
    async def get_legacy_data() -> Legacy:
        data = await db_security.execute_query_return_data('EXEC SECURITY.sps_SelectDataLegacy', fetchone=True)
        return Legacy.from_db(data)
    
    @staticmethod
    async def validate_auth_token(token_id:str, user_id:str) -> bool:
        query = """ 
        EXEC SECURITY.sps_ValidateUserToken  
        @i_idToken = :token_id,
        @i_idUser = :user_id
        """
        is_auth = await db_security.execute_query_return_first_value(query, {'token_id': token_id, 'user_id': user_id})
        if not is_auth:
            raise UnauthorizedException(message=Message.PORTAL_ACCESS_RESTRICTED_MSG, error=Message.PORTAL_ACCESS_RESTRICTED_MSG, status_code=Code.UNAUTHORIZATED_CODE)
            
        return is_auth


import re

class CaseConverter:
    
    @staticmethod
    def case_to_snake(key: str) -> str:
        key = re.sub(r'[\s\-\.]', '_', key) 
        key = re.sub(r'(.)([A-Z][a-z]+)', r'\1_\2', key)
        key = re.sub(r'([a-z0-9])([A-Z])', r'\1_\2', key)
        return key.lower()
    
    @staticmethod
    def snake_to_camel(key: str) -> str:
        components = key.split('_')
        return components[0] + ''.join(x.title() for x in components[1:])
    
    @staticmethod
    def snake_to_pascal(key: str) -> str:
        components = key.split('_')
        return ''.join(x.title() for x in components)


from functools import wraps
from starlette.datastructures import Headers
from Osdental.Models.Response import Response
from Osdental.Models.FakeInfo import FakeInfo
from Osdental.Shared.Enums.GrahpqlOperation import GraphqlOperation
from Osdental.Shared.Enums.Constant import Constant

# Client
def with_grpc_metadata(func):
    """
    Decorator to transform data to bytes and build metadata for gRPC.
    """
    @wraps(func)
    async def wrapper(self, data, headers: Headers, *args, **kwargs):
        # Transform data to bytes if it is str
        if isinstance(data, str):
            data_bytes = data.encode(Constant.DEFAULT_ENCODING)
        else:
            data_bytes = data

        # Build metadata from headers
        user_token = headers.get('authorization', '')
        token_value = user_token.split(' ')[1] if user_token.startswith('Bearer ') else user_token
        metadata = [
            ('authorization', token_value),
            ('dynamicclientid', headers.get('dynamicClientId', '')),
        ]

        # Add extra metadata if it comes in kwargs
        extra_metadata = kwargs.pop('extra_metadata', None)
        if extra_metadata:
            if isinstance(extra_metadata, dict):
                metadata.extend(extra_metadata.items())
            else:
                metadata.extend(extra_metadata)

        # Call the actual gRPC method passing request and metadata
        res = await func(self, data_bytes, metadata, *args, **kwargs)

        # Return a uniform Response
        return Response(status=res.status, message=res.message, data=res.data)

    return wrapper

# Server
def __build_grpc_context(request, context, operation_name=GraphqlOperation.QUERY) -> tuple[str, FakeInfo]:
    if isinstance(request.data, bytes):
        aes_data_str = request.data.decode(Constant.DEFAULT_ENCODING)
    else:
        aes_data_str = str(request.data)

    metadata_dict = dict(context.invocation_metadata())
    user_token = metadata_dict.get('authorization')

    context_dict = {
        'user_token': user_token,
        'headers': metadata_dict,
    }

    info = FakeInfo(context_dict, operation_name)
    return aes_data_str, info


def with_grpc_info(operation_name: GraphqlOperation = GraphqlOperation.QUERY):
    """
    Decorator to wrap gRPC methods and build the FakeInfo.
    """
    def decorator(func):
        @wraps(func)
        async def wrapper(servicer, request, context):
            aes_data_str, info = __build_grpc_context(request, context, operation_name)
            return await func(servicer, aes_data_str, info)
        return wrapper
    return decorator


import hashlib
from Osdental.Shared.Enums.Constant import Constant

class SHA512:

    @staticmethod
    def hash_password(password: str) -> str:
        """Generates the SHA-512 hash of a password."""
        hash_object = hashlib.sha512(password.encode(Constant.DEFAULT_ENCODING))
        hashed_password = hash_object.hexdigest()
        return hashed_password

    @staticmethod
    def verify_password(hash_password: str, password: str) -> bool:
        """Checks if the provided password matches the stored hash."""
        return SHA512.hash_password(password) == hash_password


from enum import StrEnum

class Constant(StrEnum):
    USER_TOKEN = 'user_token'
    AES_DATA = 'aes_data'
    DEFAULT_ENCODING = 'utf-8'
    MESSAGE_LOG_INTERNAL = 'MESSAGE_LOG_INTERNAL'
    DEFAULT_EMPTY_VALUE = '*'
    RESPONSE_TYPE_ERROR = 'ERROR'
    RESPONSE_TYPE_REQUEST = 'REQUEST'
    RESPONSE_TYPE_RESPONSE = 'RESPONSE'
    MESSAGE_LOG_EXTERNAL = 'MESSAGE_LOG_EXTERNAL'


from typing import Mapping
from datetime import datetime
from tzlocal import get_localzone
from Osdental.ServicesBus.TaskQueue import task_queue
from Osdental.Models.Response import Response
from Osdental.Shared.Enums.Code import Code
from Osdental.Shared.Enums.Message import Message
from Osdental.Shared.Enums.Constant import Constant

class OSDException(Exception):
    """ Base class for all custom exceptions. """
    def __init__(self, message:str = Message.UNEXPECTED_ERROR_MSG, error:str=None, status_code:str = Code.APP_ERROR_CODE, headers:Mapping[str,str] | None = None):
        super().__init__(message)
        self.message = message
        self.error = error
        self.headers = headers
        self.status_code = status_code
        self.local_tz = get_localzone()

    async def send_to_service_bus(self) -> None:
        """ Method to send a message to the Service Bus. """
        if self.headers:
            message_json = {
                'idMessageLog': self.headers.get('Idmessagelog'),
                'type': Constant.RESPONSE_TYPE_ERROR,
                'dateExecution': datetime.now(self.local_tz).strftime('%Y-%m-%d %H:%M:%S'),
                'httpResponseCode': self.status_code,
                'messageOut': Constant.DEFAULT_EMPTY_VALUE,
                'errorProducer': self.error if self.error else Constant.DEFAULT_EMPTY_VALUE,
                'batch': Constant.DEFAULT_EMPTY_VALUE,
                'auditLog': Constant.MESSAGE_LOG_INTERNAL
            }
            await task_queue.enqueue(message_json)

    def get_response(self) -> Response:
        return Response(status=self.status_code, message=self.message).send()

    
class UnauthorizedException(OSDException):
    def __init__(
        self, 
        message:str = Message.PORTAL_ACCESS_RESTRICTED_MSG, 
        error:str = None, 
        status_code:str = Code.UNAUTHORIZATED_CODE, 
        headers:Mapping[str,str] = None
    ):
        super().__init__(message=message, error=error, status_code=status_code, headers=headers)

class RequestDataException(OSDException):
    def __init__(
        self, 
        message:str = Message.INVALID_REQUEST_PARAMS_MSG, 
        error:str = None, 
        status_code:str = Code.INVALID_REQUEST_PARAMS_CODE, 
        headers:Mapping[str,str] = None
    ):
        super().__init__(message=message, error=error, status_code=status_code, headers=headers)

class DatabaseException(OSDException):
    def __init__(
        self, 
        message:str = Message.UNEXPECTED_ERROR_MSG, 
        error:str = None, 
        status_code:str = Code.DATABASE_ERROR_CODE, 
        headers:Mapping[str,str] = None
    ):
        super().__init__(message=message, error=error, status_code=status_code, headers=headers)

class RSAEncryptException(OSDException):
    def __init__(
        self, 
        message:str = Message.UNEXPECTED_ERROR_MSG, 
        error:str = None, 
        status_code:str = Code.RSA_ERROR_CODE, 
        headers:Mapping[str,str] = None
    ):
        super().__init__(message=message, error=error, status_code=status_code, headers=headers)

class AESEncryptException(OSDException):
    def __init__(
        self, 
        message:str = Message.UNEXPECTED_ERROR_MSG, 
        error:str = None, 
        status_code:str = Code.AES_ERROR_CODE, 
        headers:Mapping[str,str] = None
    ):
        super().__init__(message=message, error=error, status_code=status_code, headers=headers)

class JWTokenException(OSDException):
    def __init__(
        self, 
        message:str = Message.UNEXPECTED_ERROR_MSG, 
        error:str = None, 
        status_code:str = Code.JWT_ERROR_CODE, 
        headers:Mapping[str,str] = None
    ):
        super().__init__(message=message, error=error, status_code=status_code, headers=headers)

class HttpClientException(OSDException):
    def __init__(
        self, 
        message:str = Message.UNEXPECTED_ERROR_MSG, 
        error:str = None, 
        status_code:str = Code.HTTP_ERROR_CODE, 
        headers:Mapping[str,str] = None
    ):
        super().__init__(message=message, error=error, status_code=status_code, headers=headers)

class AzureException(OSDException):
    def __init__(
        self, 
        message:str = Message.UNEXPECTED_ERROR_MSG, 
        error:str = None, 
        status_code:str = Code.AZURE_ERROR_CODE, 
        headers:Mapping[str,str] = None
    ):
        super().__init__(message=message, error=error, status_code=status_code, headers=headers)

class RedisException(OSDException):
    def __init__(
        self, 
        message:str = Message.UNEXPECTED_ERROR_MSG, 
        error:str = None, 
        status_code:str = Code.REDIS_ERROR_CODE, 
        headers:Mapping[str,str] = None
    ):
        super().__init__(message=message, error=error, status_code=status_code, headers=headers)

class ValidationDataException(OSDException):
    def __init__(
        self, 
        message:str = Message.UNEXPECTED_ERROR_MSG, 
        error:str = None, 
        status_code:str = Code.REQUEST_VALIDATION_ERROR_CODE, 
        headers:Mapping[str,str] = None
    ):
        super().__init__(message=message, error=error, status_code=status_code, headers=headers)

class UnexpectedException(OSDException):
    def __init__(
        self, 
        message:str = Message.UNEXPECTED_ERROR_MSG, 
        error:str = None, 
        status_code:str = Code.APP_ERROR_CODE, 
        headers:Mapping[str,str] = None
    ):
        super().__init__(message=message, error=error, status_code=status_code, headers=headers)

class MissingFieldException(OSDException):
    def __init__(
        self, 
        message:str = Message.MISSING_FIELD_ERROR_MSG, 
        error:str = None, 
        status_code:str = Code.MISSING_FIELD_ERROR_CODE, 
        headers:Mapping[str,str] = None
    ):
        super().__init__(message=message, error=error, status_code=status_code, headers=headers)

class ProfilePermissionDeniedException(OSDException):
    def __init__(
        self,
        message: str = Message.PROFILE_PERMISSION_DENIED_MSG,
        error: str = None,
        status_code: str = Code.PROFILE_PERMISSION_DENIED_CODE,
        headers: Mapping[str, str] = None
    ):
        super().__init__(message=message, error=error, status_code=status_code, headers=headers)

class InvalidFormatException(OSDException):
    def __init__(
        self,
        message: str = Message.INVALID_FORMAT_MSG,
        error: str = None,
        status_code: str = Code.INVALID_FORMAT_CODE,
        headers: Mapping[str, str] = None
    ):
        super().__init__(message=message, error=error, status_code=status_code, headers=headers)



from Osdental.Handlers.Instances import db_security, microservice_name
from Osdental.Shared.Enums.Code import Code

class RedisTable:

    @staticmethod
    async def create_redis_key(user_id: str, method_controller: str, key: str) -> None:
        query = """ 
        EXEC SECURITY.spi_InsertRedisTable
        @i_idUser = :user_id,
        @i_microService = :microservice,
        @i_methodController = :method_controller,
        @i_keyName = :key_name
        """
        params = {
            'user_id': user_id,
            'microservice': microservice_name,
            'method_controller': method_controller,
            'key_name': key
        }
        await db_security.execute_query(query, params, Code.PROCESS_SUCCESS_CODE)

    @staticmethod
    async def delete_redis_key(key: str) -> None:
        query = """ 
        EXEC SECURITY.spd_DeleteRedisTable
        @i_keyName = :key_name
        """
        params = {
            'key_name': key
        }
        await db_security.execute_query(query, params, Code.PROCESS_SUCCESS_CODE)


from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.backends import default_backend
from typing import Dict
import jwt
from datetime import datetime, timezone, timedelta
from Osdental.Handlers.DBCatalogQuery import DBCatalogQuery
from Osdental.Handlers.DBConnection import DBConnection
from Osdental.Shared.Enums.Message import Message
from Osdental.Exception.ControlledException import MissingFieldException

class CData:
    @staticmethod
    async def generate_token_initial():    
        return await CData.__generate_jwt()

    @staticmethod
    async def generate_token_account(sub_account: str = None):    
        return await CData.__generate_jwt(sub_account)

    @staticmethod
    async def __generate_jwt(sub_account: str = None) -> Dict[str, str]:        
        if sub_account is None or sub_account != '':                        
            now = datetime.now(timezone.utc)            
            iat = int(now.timestamp())                        
            catalog = await DBCatalogQuery.get_catalog_data('CDataIntegration')     
            data_credentials = await DBConnection.get_data_credentials('CData')                                     

            if catalog and data_credentials:
                exp = int((now + timedelta(minutes=data_credentials.get('expToken'))).timestamp())            
                        
                payload = {
                    'tokenType': catalog.get('tokenType'),
                    'iss': catalog.get('ISS'),
                    'iat': iat,
                    'exp': exp
                }            
                if sub_account:
                    payload['sub'] = sub_account
                private_key = CData.generate_private_key(data_credentials.get('keyPrivate'))            
                token = jwt.encode(payload, private_key, algorithm='RS256')                                                                    
                return token
            else:
                raise MissingFieldException(message=Message.CATALOG_DATA_CREDENTIALS_MISSED)
        else:             
            raise MissingFieldException(message=Message.SUB_ACCOUNT_REQUIRED)

    @staticmethod
    def generate_private_key(private_rsa: str):                  
        private_key = serialization.load_pem_private_key(
            private_rsa.encode(),
            password=None,
            backend=default_backend()
        )
        return private_key    


from typing import List, Dict
from Osdental.Handlers.Instances import db_catalog
from Osdental.Models.Catalog import Catalog

class DBCatalogQuery:
    
    @staticmethod
    async def get_catalog_data(catalog_name: str, raw: bool = False) -> Dict[str,str] | List[Dict[str,str]]:
        rows = await db_catalog.execute_query_return_data('EXEC CATALOG.sps_GetCatalogByName @i_nameCatalog = :catalog_name', {'catalog_name': catalog_name})
        if raw:
            return rows
        
        items = [Catalog.from_db(row) for row in rows if row.get('value')]
        return {item.code: item.value for item in items if item.value is not None}


from contextlib import asynccontextmanager
from typing import Dict, List, Tuple, AsyncGenerator
from sqlalchemy import text
from sqlalchemy.ext.asyncio import create_async_engine, AsyncSession
from sqlalchemy.orm import sessionmaker
from Osdental.Exception.ControlledException import DatabaseException
from Osdental.Shared.Utils.DataUtils import DataUtils
from Osdental.Shared.Enums.Code import Code
from Osdental.Shared.Enums.Message import Message

class Connection:

    _instances: Dict[str, 'Connection'] = {}

    def __new__(cls, db_url: str):
        if db_url not in cls._instances:
            cls._instances[db_url] = super(Connection, cls).__new__(cls)
        return cls._instances[db_url]

    def __init__(self, db_url: str):
        if not hasattr(self, 'initialized'):
            self.engine = create_async_engine(
                db_url,
                pool_size=20,          
                max_overflow=40,       
                pool_timeout=30,       
                pool_recycle=3600     
            )
            self.session_factory = sessionmaker(
                bind=self.engine, 
                class_=AsyncSession, 
                expire_on_commit=False
            )
            self.initialized = True

    def get_session(self) -> AsyncSession:
        return self.session_factory()


    async def execute_query_return_first_value(self, query:str, params:Dict[str,str] = None) -> str:
        try:
            async with self.get_session() as session:
                async with session.begin():
                    result = await session.execute(text(query), params)
                    value = result.scalar()
                    return DataUtils.normalize_uuid_value(value)
        except DatabaseException:
            raise
        except Exception as e:
            raise DatabaseException(message=Message.DATABASE_EXECUTION_ERROR_MSG, error=str(e))


    async def execute_query_return_data(self, query:str, params:Dict[str,str] = None, fetchone=False) -> List[Dict[str,str]] | Dict[str,str]:
        try:
            async with self.get_session() as session:
                async with session.begin():
                    result = await session.execute(text(query), params)
                    keys = result.keys()
                    if fetchone:
                        row = result.fetchone()
                        return DataUtils.normalize_uuids_dict(dict(zip(keys, row))) if row else {}
                    
                    rows = result.fetchall()
                    return [DataUtils.normalize_uuids_dict(dict(zip(keys, row))) for row in rows] if rows else []
        except DatabaseException:
            raise
        except Exception as e:
            raise DatabaseException(message=Message.DATABASE_EXECUTION_ERROR_MSG, error=str(e))


    async def execute_query_return_message(self, query:str, params:Dict[str,str] = None, code:str | Tuple[str,str] = Code.PROCESS_SUCCESS_CODE) -> str:
        try:
            async with self.get_session() as session:
                async with session.begin():
                    result = await session.execute(text(query), params)
                    row = result.fetchone() 
                    if not row:
                        raise DatabaseException(message=Message.NO_RESULTS_FOUND_MSG, error=Message.NO_RESULTS_FOUND_MSG)
                    
                    status_code = (code,) if isinstance(code, str) else code
                    if row.STATUS_CODE not in status_code:
                        raise DatabaseException(message=row.STATUS_MESSAGE, error=row.STATUS_MESSAGE, status_code=row.STATUS_CODE)

                    return row.STATUS_MESSAGE
        except DatabaseException:
            raise
        except Exception as e:
            raise DatabaseException(message=Message.DATABASE_EXECUTION_ERROR_MSG, error=str(e))


    async def execute_query(self, query:str, params:Dict[str,str] = None, code:str | Tuple[str,str] = None) -> None | Dict[str,str]:
        try:
            async with self.get_session() as session:
                async with session.begin():
                    result = await session.execute(text(query), params)
                    if code:
                        row = result.fetchone()
                        if not row:
                            raise DatabaseException(message=Message.NO_RESULTS_FOUND_MSG, error=Message.NO_RESULTS_FOUND_MSG)
                        
                        status_code = (code,) if isinstance(code, str) else code
                        if row.STATUS_CODE not in status_code:
                            raise DatabaseException(message=row.STATUS_MESSAGE, error=row.STATUS_MESSAGE, status_code=row.STATUS_CODE)

                        return DataUtils.normalize_uuids_dict(dict(zip(result.keys(), row)))
                    
                    return None
        except DatabaseException:
            raise
        except Exception as e:
            raise DatabaseException(message=Message.DATABASE_EXECUTION_ERROR_MSG, error=str(e))
            
    
    async def execute_transaction_queries(self, data: List[Dict[str, str]], return_data: bool = False) -> Dict[str, str] | str | None:
        try:
            result_data = None
            async with self.get_session() as session:
                async with session.begin():
                    for item in data:
                        query = item.get('query')
                        params = item.get('params', {})
                        codes = item.get('code', Code.PROCESS_SUCCESS_CODE)

                        if not query:
                            raise DatabaseException(
                                message=Message.QUERY_NOT_PROVIDED_MSG,
                                error=Message.QUERY_NOT_PROVIDED_MSG
                            )

                        result = await session.execute(text(query), params)
                        row = result.fetchone()
                        keys = result.keys()
                        result.close()

                        if not row:
                            raise DatabaseException(
                                message=Message.NO_RESULTS_FOUND_MSG,
                                error=Message.NO_RESULTS_FOUND_MSG
                            )

                        accepted_codes = (codes,) if isinstance(codes, str) else codes
                        if row.STATUS_CODE not in accepted_codes:
                            raise DatabaseException(
                                message=row.STATUS_MESSAGE,
                                error=row.STATUS_MESSAGE,
                                status_code=row.STATUS_CODE
                            )

                        if return_data:
                            result_data = DataUtils.normalize_uuids_dict(dict(zip(keys, row)))
                        else:
                            result_data = row.STATUS_MESSAGE

            return result_data
        except DatabaseException:
            raise
        except Exception as e:
            raise DatabaseException(message=Message.DATABASE_EXECUTION_ERROR_MSG, error=str(e))

    @asynccontextmanager
    async def get_transaction_session(self) -> AsyncGenerator[AsyncSession, None]:
        """Returns a session with active transaction for scope controlled by the caller."""
        async with self.get_session() as session:
            async with session.begin():
                try:
                    yield session
                except Exception:
                    await session.rollback()
                    raise
                else:
                    await session.commit()

    async def execute_with_session(self, session: AsyncSession, query: str, params: Dict[str, str] = None):
        """Executes a query using an already open external session (transaction controlled outside)."""
        result = await session.execute(text(query), params)
        return result
    
    async def close_engine(self) -> None:
        """Dispose of the engine and remove instance."""
        if self.engine:
            await self.engine.dispose() 
        
        if self.db_url in self._instances:
            del self._instances[self.db_url]


import random
import string
from Osdental.Shared.Enums.Message import Message

class PasswordGenerator:

    @staticmethod
    def generate_simple(length: int = 12) -> str:
        valid_characters = string.ascii_letters + string.digits + '!@#$%^&*()-_=+[]{}|:,.?'
        password = ''.join(random.choice(valid_characters) for _ in range(length))
        return password

    @staticmethod
    def generate(length: int = 12, use_uppercase: bool = True, use_digits: bool = True, use_symbols: bool = True, exclude_ambiguous: bool = True) -> str:
        lowercase = string.ascii_lowercase
        uppercase = string.ascii_uppercase if use_uppercase else ''
        digits = string.digits if use_digits else ''
        symbols = string.punctuation if use_symbols else ''

        if exclude_ambiguous:
            ambiguous = 'Il1O0|'
            lowercase = ''.join(c for c in lowercase if c not in ambiguous)
            uppercase = ''.join(c for c in uppercase if c not in ambiguous)
            digits = ''.join(c for c in digits if c not in ambiguous)
            symbols = ''.join(c for c in symbols if c not in ambiguous)

        all_chars = lowercase + uppercase + digits + symbols
        if not all_chars:
            raise ValueError(Message.NO_PASSWORD_CHARACTERS_MSG)

        return ''.join(random.choice(all_chars) for _ in range(length))

    @staticmethod
    def generate_with_rules(min_upper: int = 1, min_digits: int = 1, min_symbols: int = 1, length: int = 12) -> str:
        if length < (min_upper + min_digits + min_symbols):
            raise ValueError(Message.INSUFFICIENT_LENGTH_MSG)

        password = []
        password.extend(random.choices(string.ascii_uppercase, k=min_upper))
        password.extend(random.choices(string.digits, k=min_digits))
        password.extend(random.choices(string.punctuation, k=min_symbols))
        password.extend(random.choices(string.ascii_lowercase, k=length - len(password)))

        random.shuffle(password)
        return ''.join(password)



import logging
import colorlog

handler = colorlog.StreamHandler()
handler.setFormatter(colorlog.ColoredFormatter(
    '%(log_color)s%(levelname)s:%(name)s:%(message)s',  
    log_colors={
        'DEBUG': 'cyan',
        'INFO': 'green',
        'WARNING': 'yellow',
        'ERROR': 'red',
        'CRITICAL': 'bold_red',
    }
))
logger = colorlog.getLogger('my_logger')
logger.addHandler(handler)
logger.setLevel(logging.DEBUG)


from __future__ import annotations
from dataclasses import dataclass
from typing import Optional, Dict
from Osdental.Models.Legacy import Legacy
from Osdental.Shared.Utils.CaseConverter import CaseConverter
from Osdental.Exception.ControlledException import MissingFieldException

@dataclass
class AuthToken:
    id_token: str
    id_user: str
    id_external_enterprise: str
    id_profile: str
    id_legacy: str
    id_item_report: str
    id_enterprise: str
    id_authorization: str
    user_full_name: str
    abbreviation: str
    aes_key_auth: str
    base_id_external_enterprise: Optional[str] = None
    mk_id_external_enterprise: Optional[str] = None
    jwt_user_key: Optional[str] = None
    legacy: Optional[Legacy] = None

    def __post_init__(self):
        required_fields = [
            'id_token', 'id_user', 'id_external_enterprise', 'id_profile',
            'id_legacy', 'id_item_report', 'id_enterprise', 'id_authorization',
            'user_full_name', 'abbreviation', 'aes_key_auth'
        ]
        missing = [f for f in required_fields if not getattr(self, f)]
        if missing:
            raise MissingFieldException(error=f"Missing required fields: {', '.join(missing)}")

        
    @classmethod
    def from_jwt(cls, payload: Dict[str,str]) -> AuthToken:
        mapped = {CaseConverter.case_to_snake(key): value for key, value in payload.items()}
        valid_fields = cls.__dataclass_fields__.keys()
        clean = {k: v for k, v in mapped.items() if k in valid_fields}
        return cls(**clean)


from datetime import datetime, date, timedelta
import calendar

class DateUtils:

    @staticmethod
    def now() -> datetime:
        """Returns the current date and time."""
        return datetime.now()

    @staticmethod
    def today() -> date:
        """Returns the current date."""
        return date.today()

    @staticmethod
    def current_year() -> int:
        """Returns the current year as an integer."""
        return date.today().year

    @staticmethod
    def current_month() -> int:
        """Returns the current month as an integer."""
        return date.today().month

    @staticmethod
    def parse_date(text: str, fmt: str = "%Y-%m-%d") -> date:
        """Parses a date string into a `date` object using the specified format."""
        return datetime.strptime(text, fmt).date()

    @staticmethod
    def parse_datetime(text: str, fmt: str = "%Y-%m-%d %H:%M:%S") -> datetime:
        """Parses a datetime string into a `datetime` object using the specified format."""
        return datetime.strptime(text, fmt)

    @staticmethod
    def format_date(d: date, fmt: str = "%Y-%m-%d") -> str:
        """Formats a `date` object into a string using the specified format."""
        return d.strftime(fmt)

    @staticmethod
    def format_datetime(dt: datetime, fmt: str = "%Y-%m-%d %H:%M:%S") -> str:
        """Formats a `datetime` object into a string using the specified format."""
        return dt.strftime(fmt)

    @staticmethod
    def add_days(d: date, days: int) -> date:
        """Returns a new date by adding the specified number of days."""
        return d + timedelta(days=days)

    @staticmethod
    def subtract_days(d: date, days: int) -> date:
        """Returns a new date by subtracting the specified number of days."""
        return d - timedelta(days=days)

    @staticmethod
    def days_between(start: date, end: date) -> int:
        """Returns the number of days between two dates."""
        return (end - start).days

    @staticmethod
    def is_past(d: date) -> bool:
        """Returns True if the given date is in the past."""
        return d < date.today()

    @staticmethod
    def is_future(d: date) -> bool:
        """Returns True if the given date is in the future."""
        return d > date.today()

    @staticmethod
    def is_today(d: date) -> bool:
        """Returns True if the given date is today."""
        return d == date.today()

    @staticmethod
    def start_of_month(d: date) -> date:
        """Returns the first day of the month for the given date."""
        return d.replace(day=1)

    @staticmethod
    def end_of_month(d: date) -> date:
        """Returns the last day of the month for the given date."""
        last_day = calendar.monthrange(d.year, d.month)[1]
        return d.replace(day=last_day)

    @staticmethod
    def start_of_year(d: date) -> date:
        """Returns the first day of the year for the given date."""
        return d.replace(month=1, day=1)

    @staticmethod
    def end_of_year(d: date) -> date:
        """Returns the last day of the year for the given date."""
        return d.replace(month=12, day=31)

    @staticmethod
    def to_iso_string(dt: datetime) -> str:
        """Converts a `datetime` object to ISO 8601 format string."""
        return dt.isoformat()

    @staticmethod
    def from_iso_string(text: str) -> datetime:
        """Parses an ISO 8601 format string into a `datetime` object."""
        return datetime.fromisoformat(text)

    @staticmethod
    def weekday_name(d: date) -> str:
        """Returns the name of the weekday for the given date (e.g., 'Monday')."""
        return d.strftime("%A")

    @staticmethod
    def month_name(d: date) -> str:
        """Returns the full name of the month for the given date (e.g., 'January')."""
        return d.strftime("%B")

    @staticmethod
    def get_ttl_for_midnight() -> int:
        """Returns the number of seconds remaining until the next midnight."""
        now = datetime.now()
        tomorrow = now + timedelta(days=1)
        midnight = datetime(year=tomorrow.year, month=tomorrow.month, day=tomorrow.day)
        ttl = (midnight - now).seconds
        return ttl



import random
import string

class CodeGenerator:

    @staticmethod
    def generate_numeric_code(length:int = 6) -> str:
        nums = string.digits        
        return ''.join(random.choices(nums,k=length)) 
    
    @staticmethod
    def generate_alpha_code(length:int = 6) -> str:
        return ''.join(random.choices(string.ascii_letters, k=length))
    
    @staticmethod
    def generate_uppercase_code(length:int = 6) -> str:
        return ''.join(random.choices(string.ascii_uppercase, k=length))
    
    @staticmethod
    def generate_lowercase_code(length:int = 6) -> str:
        return ''.join(random.choices(string.ascii_lowercase, k=length))
    
    @staticmethod
    def generate_alphanumeric_code(length: int = 6) -> str:
        chars = string.ascii_letters + string.digits  
        return ''.join(random.choices(chars, k=length))
    
    @staticmethod
    def generate_secure_code(length: int = 6) -> str:
        chars = string.ascii_letters + string.digits + string.punctuation
        return ''.join(random.choices(chars, k=length))


from typing import Dict, Any
from contextlib import asynccontextmanager
from sqlalchemy.ext.asyncio import AsyncSession
from Osdental.Exception.ControlledException import DatabaseException
from Osdental.Shared.Enums.Message import Message

class UnitOfWork:
    def __init__(self, session: AsyncSession, repositories: Dict[str, Any] = None):
        """
        :param session: session SQLAlchemy
        :param repositories: dict with name: repository class
        """
        self.session = session
        self._repositories = repositories or {}

    def __getattr__(self, item):
        """
        Allows dynamic access to repositories
        """
        if item in self._repositories:
            return self._repositories[item]
        raise AttributeError(f"'UnitOfWork' object has no attribute '{item}'")

    @asynccontextmanager
    async def __call__(self):
        try:
            async with self.session.begin():
                # Dynamically initialize repositories with the session
                for name, repo_cls in self._repositories.items():
                    self._repositories[name] = repo_cls(self.session)
                try:
                    yield self
                except Exception:
                    await self.session.rollback()
                    raise
                else:
                    await self.session.commit()
        except Exception as err:
            raise DatabaseException(message=Message.DATABASE_EXECUTION_ERROR_MSG, error=str(err))



from enum import StrEnum

class Code(StrEnum):
    PROCESS_SUCCESS_CODE = 'DB_0004'
    UNAUTHORIZATED_CODE = 'DB_ERROR_0006'
    REQUEST_NOT_COMPLETED_CODE = 'DB_ERROR_9999'
    DATA_NOT_FOUND_CODE = 'DB_ERROR_0404'
    APP_ERROR_CODE = 'DB_ERROR_APP'
    INVALID_REQUEST_PARAMS_CODE = 'DB_WARNING_INVALID_REQUEST_PARAMS'
    DATABASE_ERROR_CODE = 'DB_WARNING_DATABASE_ERROR'
    RSA_ERROR_CODE = 'DB_ERROR_RSA'
    AES_ERROR_CODE = 'DB_ERROR_AES'
    JWT_ERROR_CODE = 'DB_ERROR_JWT'
    HTTP_ERROR_CODE = 'DB_ERROR_HTTP'
    AZURE_ERROR_CODE = 'DB_ERROR_AZURE'
    REDIS_ERROR_CODE = 'DB_ERROR_REDIS'
    REQUEST_VALIDATION_ERROR_CODE = 'DB_WARNING_REQUEST_VALIDATION'
    MISSING_FIELD_ERROR_CODE = 'DB_WARNING_MISSING_FIELD'
    PROFILE_PERMISSION_DENIED_CODE = 'DB_WARNING_PROFILE_PERMISSION_DENIED'
    INVALID_FORMAT_CODE = 'DB_WARNING_INVALID_FORMAT'


import jwt
from typing import Dict
from Osdental.Exception.ControlledException import JWTokenException
from Osdental.Shared.Logger import logger
from Osdental.Shared.Enums.Message import Message

class JWT:

    @staticmethod
    def generate_token(payload:Dict[str,str], jwt_secret_key:str) -> str:
        try:
            token = jwt.encode(payload, jwt_secret_key, algorithm='HS256')
            return token

        except Exception as e:
            logger.error(f'Unexpected jwt generating error: {str(e)}')
            raise JWTokenException(message=Message.UNEXPECTED_ERROR_MSG, error=str(e))


    @staticmethod
    def extract_payload(jwt_token:str, jwt_secret_key:str) -> Dict[str,str]:
        try:
            payload = jwt.decode(jwt_token, jwt_secret_key, algorithms=['HS256'])
            return payload

        except Exception as e:
            logger.error(f'Unexpected jwt extract payload error: {str(e)}')
            raise JWTokenException(message=Message.UNEXPECTED_ERROR_MSG, error=str(e)) 


from typing import Dict, Type, TypeVar, get_origin, get_args, Union, Any
from dataclasses import fields, is_dataclass
from Osdental.Shared.Utils.CaseConverter import CaseConverter
from Osdental.Exception.ControlledException import OSDException

T = TypeVar('T')

class Mapper:

    @staticmethod
    def map_data(data: Dict[str, Any], model: Type[T]) -> T:
        if not is_dataclass(model):
            raise OSDException(error=f'Target model {model} is not a dataclass', message='Mapping error')

        mapped = {CaseConverter.case_to_snake(k): v for k, v in data.items()}
        instance_data = {}
        extra = {}

        for field in fields(model):
            field_name = field.name
            field_type = field.type
            value = mapped.pop(field_name, None) 
            if value is not None:
                instance_data[field_name] = Mapper._map_value(value, field_type)

        if 'extra' in [f.name for f in fields(model)]:
            extra = mapped
            instance_data['extra'] = extra

        return model(**instance_data)

    @staticmethod
    def _map_value(value: Any, field_type: Any):
        origin = get_origin(field_type)
        args = get_args(field_type)

        if origin is Union and type(None) in args:
            actual_type = next((arg for arg in args if arg is not type(None)), None)
            return Mapper._map_value(value, actual_type)

        if origin is list:
            item_type = args[0]
            if isinstance(value, list):
                return [
                    Mapper._map_value(item, item_type)
                    for item in value
                ]
            return value 

        if is_dataclass(field_type) and isinstance(value, dict):
            return Mapper.map_data(value, field_type)

        return value



from __future__ import annotations
from dataclasses import dataclass
from typing import Dict
from Osdental.Shared.Utils.CaseConverter import CaseConverter
from Osdental.Exception.ControlledException import MissingFieldException
from Osdental.Shared.Enums.Message import Message

@dataclass
class CdataIntegration:
    id_cdata_integration: str
    exp_token: int
    key_private: str

    def __post_init__(self):
        if not self.id_cdata_integration:
            raise MissingFieldException(message=Message.ID_CDATA_INTEGRATION_REQUIRED)
        
        if not self.exp_token:
            raise MissingFieldException(message=Message.EXP_TIME_REQUIRED)                
        
        if not self.key_private:
            raise MissingFieldException(message=Message.KEY_PRIVATE_REQUIRED)
    
    @classmethod
    def from_db(cls, record: Dict[str,str]) -> CdataIntegration:
        mapped = {CaseConverter.case_to_snake(key) : value for key, value in record.items()}
        valid_fields = cls.__dataclass_fields__.keys()
        clean = {k: v for k, v in mapped.items() if k in valid_fields}
        return cls(**clean)

@dataclass
class CdataInfo:
    token_type: str
    iss: str

    @classmethod
    def from_db(cls, record: Dict[str,str]) -> CdataInfo:
        mapped = {CaseConverter.case_to_snake(key) : value for key, value in record.items()}
        valid_fields = cls.__dataclass_fields__.keys()
        clean = {k: v for k, v in mapped.items() if k in valid_fields}
        return cls(**clean)


import json
from typing import Dict
from azure.servicebus.aio import ServiceBusClient
from azure.servicebus import ServiceBusMessage
from Osdental.Shared.Logger import logger
from Osdental.Shared.Enums.Message import Message

class ServicesBus:

    def __init__(self, conn_str:str, queue_name:str):
        self.conn_str = conn_str
        self.queue_name = queue_name


    async def send_message(self, message:Dict[str,str] | str) -> None:
        """Method to send a message to the Service Bus."""
        try:
            data = json.dumps(message) if isinstance(message, dict) else message
            async with ServiceBusClient.from_connection_string(self.conn_str) as servicebus_client:
                async with servicebus_client.get_queue_sender(queue_name=self.queue_name) as sender:
                    message = ServiceBusMessage(data)
                    await sender.send_messages(message)

        except Exception as e:
            logger.error(f'Unexpected services bus error: {str(e)}')
            raise ValueError(f'{Message.UNEXPECTED_ERROR_MSG}: {str(e)}')



from __future__ import annotations
from dataclasses import dataclass
from typing import Dict
from Osdental.Shared.Utils.CaseConverter import CaseConverter

@dataclass
class Catalog:
    id_catalog: str
    name_catalog: str
    id_detail: str
    code: str
    value: str

    @classmethod
    def from_db(cls, record: Dict[str,str]) -> Catalog:
        mapped = {CaseConverter.case_to_snake(key) : value for key, value in record.items()}
        valid_fields = cls.__dataclass_fields__.keys()
        clean = {k: v for k, v in mapped.items() if k in valid_fields}
        return cls(**clean)


import os
from dotenv import load_dotenv
from Osdental.Database.Connection import Connection
from Osdental.Encryptor.Aes import AES

load_dotenv(dotenv_path='.env', override=True)

db_catalog = Connection(os.getenv('DATABASE_CATALOG'))
db_security = Connection(os.getenv('DATABASE_SECURITY'))
db_connection = Connection(os.getenv('DATABASE_CONNECTION'))
microservice_name = os.getenv('MICROSERVICE_NAME')
jwt_user_key = os.getenv('JWT_USER_KEY')
aes = AES()
environment = os.getenv('ENVIRONMENT')
microservice_name = os.getenv('MICROSERVICE_NAME')
microservice_version = os.getenv('MICROSERVICE_VERSION')


from types import SimpleNamespace
from Osdental.Shared.Enums.GrahpqlOperation import GraphqlOperation

class FakeInfo:
    
    def __init__(self, context: dict, operation_name=GraphqlOperation.QUERY):
        self.context = context
        self.operation = SimpleNamespace(operation=operation_name)


from typing import Dict
from Osdental.Handlers.Instances import db_connection

class DBConnection:
    
    @staticmethod
    async def get_data_credentials(provider_name: str) -> Dict[str,str]:        
        return await db_connection.execute_query_return_data('EXEC CONNECT.sps_GetCDataCredentials @i_providerName = :provider_name', {'provider_name': provider_name}, fetchone=True)


from typing import Mapping, Dict
from json import dumps
from fastapi import Response, BackgroundTasks
from datetime import datetime
from tzlocal import get_localzone
from Osdental.ServicesBus.TaskQueue import task_queue
from Osdental.Shared.Enums.Constant import Constant

class CustomResponse(Response):

    def __init__(self, content: Dict[str,str] | str | None, status_code: int = 200, headers: Mapping[str, str] | None = None, media_type: str | None = None, background: BackgroundTasks | None = None, batch: str | None = None ):
        """ Custom Response constructor for FastAPI """
        self.content = content 
        self.local_tz = get_localzone()
        self.batch = batch

        # Use FastAPI Response constructor for standard attributes (status_code, media_type, etc.)
        super().__init__(
            content=content,
            status_code=status_code,
            headers=headers,
            media_type=media_type,
            background=background,
        )

    async def send_to_service_bus(self) -> None:
        """ Send the response to the Service Bus asynchronously """
        id_message_log = self.headers.get('Idmessagelog') 
        message_json = {
            'idMessageLog': id_message_log,
            'type': 'RESPONSE',
            'dateExecution': datetime.now(self.local_tz).strftime('%Y-%m-%d %H:%M:%S'),
            'httpResponseCode': str(self.status_code),
            'messageOut': dumps(self.content) if isinstance(self.content, dict) else self.content,
            'batch': self.batch if self.batch else Constant.DEFAULT_EMPTY_VALUE,
            'errorProducer': Constant.DEFAULT_EMPTY_VALUE,
            'auditLog': Constant.MESSAGE_LOG_INTERNAL
        }
        await task_queue.enqueue(message_json)



from setuptools import setup, find_packages
 
setup(
    name="Encryptors", 
    version="1.8.20",
    author="OSDental LLC",
    author_email="support@osdental.ai",
    description="End-to-end algorithm library",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    package_dir={"": "src"},  
    packages=find_packages(where="src"), 
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.10", 
    install_requires=[
        "aiohttp==3.11.12",
        "annotated-types==0.7.0",
        "anyio==4.7.0",
        "ariadne==0.26.2",
        "azure-core==1.32.0",
        "azure-servicebus==7.13.0",
        "azure-storage-blob==12.24.0",
        "certifi==2024.12.14",
        "cffi==1.17.1",
        "charset-normalizer==3.4.1",
        "cryptography==44.0.0",
        "fastapi==0.115.6",
        "greenlet==3.1.1",
        "h11==0.14.0",
        "httpcore==1.0.7",
        "httpx==0.28.1",
        "idna==3.10",
        "isodate==0.7.2",
        "pycparser==2.22",
        "pydantic==2.10.4",
        "pydantic_core==2.27.2",
        "PyJWT==2.10.1",
        "aioodbc==0.5.0",
        "python-dotenv==1.0.1",
        "requests==2.32.3",
        "six==1.17.0",
        "sniffio==1.3.1",
        "SQLAlchemy==2.0.36",
        "starlette==0.41.3",
        "tenacity==9.1.2",
        "typing_extensions==4.12.2",
        "tzlocal==5.2",
        "urllib3==2.3.0",
        "redis==5.2.1",
        "colorlog==6.9.0",
        "click==8.2.0"
    ],
    entry_points={
    'console_scripts': [
        'osdcli = Osdental.Cli:cli',  # <nombre> = <paquete>.<archivo>:<función>
    ]
},
)



import os
import mimetypes
from Osdental.Exception.ControlledException import ValidationDataException
from Osdental.Shared.Message import Message
from Osdental.Shared.Enums.FileType import FileType

class FileMetaData:
    def __init__(self, file_path: str):
        self.file_path = file_path
        self.name = ''
        self.ext = ''
        self.mime_type = 'application/octet-stream'
        self.type = 'other'
        self._extract_metadata()

    def _extract_metadata(self):
        if not self.file_path:
            raise ValidationDataException(error=Message.FILE_PATH_NOT_PROVIDED_MSG)

        self.name, ext = os.path.splitext(os.path.basename(self.file_path))
        self.ext = ext.lstrip('.')
        self.mime_type = mimetypes.guess_type(self.file_path)[0] or self.mime_type
        self.type = self._get_general_type(self.mime_type)

    def _get_general_type(self, mime_type: str) -> str:
        if mime_type.startswith('image/'):
            return FileType.IMAGE
        elif mime_type.startswith('video/'):
            return FileType.VIDEO
        elif mime_type.startswith('audio/'):
            return FileType.AUDIO
        elif mime_type in [
            'application/pdf',
            'application/msword',
            'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
            'text/plain'
        ]:
            return FileType.DOCUMENT
        elif mime_type in ['application/zip', 'application/x-rar-compressed']:
            return FileType.ARCHIVE
        else:
            return FileType.OTHER

    def to_dict(self):
        return {
            'name': self.name,
            'ext': self.ext,
            'type': self.type,
            'mimeType': self.mime_type
        }



from enum import StrEnum

class Message(StrEnum):
    UNEXPECTED_ERROR_MSG = 'Something went wrong while processing your request. Please try again later.'
    PORTAL_ACCESS_RESTRICTED_MSG = 'You are not authorized to access this portal.'
    PROCESS_SUCCESS_MSG = 'Process executed successfully.'
    NO_RESULTS_FOUND_MSG = 'No records were found matching your request.'
    INVALID_REQUEST_PARAMS_MSG = 'Please review the required fields and try again.'
    SERVER_NETWORK_ACCESS_ERROR_MSG = 'Error making the server accessible on the network.'
    HEXAGONAL_SERVICE_CREATED_MSG = 'The hexagonal service structure was created.'
    PYCACHE_CLEANUP_SUCCESS_MSG = 'All __pycache__ have been removed.'
    NO_PASSWORD_CHARACTERS_MSG = 'There are no characters available to generate the password.'
    INSUFFICIENT_LENGTH_MSG = 'Insufficient length to meet minimum rules.'
    LEGACY_NAME_REQUIRED_MSG = 'Legacy name cannot be empty.'
    REFRESH_TOKEN_EXP_REQUIRED_MSG = 'Refresh token expiration time cannot be empty.'
    ACCESS_TOKEN_EXP_REQUIRED_MSG = 'Access token expiration time cannot be empty.'
    PUBLIC_KEY2_REQUIRED_MSG = 'Public key 2 cannot be empty.'
    PRIVATE_KEY1_REQUIRED_MSG = 'Private key 1 cannot be empty.'
    PRIVATE_KEY2_REQUIRED_MSG = 'Private key 2 cannot be empty.'
    AES_KEY_USER_REQUIRED_MSG = 'AES key user cannot be empty.'
    AES_KEY_AUTH_REQUIRED_MSG = 'AES key auth cannot be empty.'
    MISSING_FIELD_ERROR_MSG = 'A required field is missing. Please review the data.'
    ID_CDATA_INTEGRATION_REQUIRED = 'Cdata integration cannot be empty or null'
    EXP_TIME_REQUIRED = 'Expiration time (cdata integration) cannot be empty or null'
    KEY_PRIVATE_REQUIRED = 'Key private cannot be empty or null'
    SUB_ACCOUNT_REQUIRED = 'Sub account parameter is required'
    CATALOG_DATA_CREDENTIALS_MISSED = 'Catalog data or/and data credentials not found'
    PROFILE_PERMISSION_DENIED_MSG = 'Your profile does not have permission to perform this action.'
    INVALID_FORMAT_MSG = 'The provided data format is invalid.'
    INVALID_AES_JSON_FORMAT_MSG = 'Invalid JSON format in AES decrypted data.'
    UNEXPECTED_DECRYPTED_DATA_FORMAT_MSG = 'Unexpected format in decrypted data.'
    REDIS_CLEANUP_SUCCESS_MSG = 'Redis cleanup completed successfully.'
    REDIS_CLEANUP_ERROR_MSG = 'An error occurred while attempting to clean up Redis.'
    DATABASE_EXECUTION_ERROR_MSG = 'An unexpected error occurred while executing a database operation.'
    QUERY_NOT_PROVIDED_MSG = 'Query not provided. Please include a valid query in your request.'
    FILE_PATH_NOT_PROVIDED_MSG = 'File path not provided.'
    PROTO_FILES_GENERATED_MSG = 'Proto files have been successfully generated.'


import os
import sys
import subprocess
import platform
import click
from Osdental.Shared.Logger import logger
from Osdental.Shared.Enums.Message import Message
from Osdental.Shared.Utils.CaseConverter import CaseConverter

SRC_PATH = 'src'
APP_PATH = os.path.join(SRC_PATH, 'Application')
DOMAIN_PATH = os.path.join(SRC_PATH, 'Domain')
INFRA_PATH = os.path.join(SRC_PATH, 'Infrastructure')
GRAPHQL_PATH = os.path.join(INFRA_PATH, 'Graphql')
GRPC_PATH = os.path.join(INFRA_PATH, 'Grpc')
RESOLVERS_PATH = os.path.join(GRAPHQL_PATH, 'Resolvers')
SCHEMAS_PATH = os.path.join(GRAPHQL_PATH, 'Schemas')

@click.group()
def cli():
    """Comandos personalizados para gestionar el proyecto."""
    pass

@cli.command()
def clean():
    """Borrar todos los __pycache__."""
    if platform.system() == 'Windows':
        subprocess.run('for /d /r . %d in (__pycache__) do @if exist "%d" rd /s/q "%d"', shell=True)
    else:
        subprocess.run("find . -name '__pycache__' -type d -exec rm -rf {} +", shell=True)

    logger.info(Message.PYCACHE_CLEANUP_SUCCESS_MSG)


@cli.command(name='start-app')
@click.argument('app')
def start_app(app: str):
    """Crear un servicio con estructura hexagonal y CRUD básico."""
    app = CaseConverter.snake_to_pascal(app)
    app_upper = app.upper()
    if '-' in app:
        part_one, part_two = tuple(app.split('-'))
        app = part_one + part_two
        app_upper = f'{part_one}_{part_two}'.upper()

    name_method = CaseConverter.case_to_snake(app)
    data = 'data: Dict[str,str]'
    token = 'token: AuthToken'
    api_type_response = 'Response!'
    
    directories = [
        os.path.join(SRC_PATH),
        os.path.join(APP_PATH, 'UseCases'),
        os.path.join(APP_PATH, 'Interfaces'),
        os.path.join(DOMAIN_PATH, 'Interfaces'),
        os.path.join(DOMAIN_PATH, 'Models'),
        os.path.join(RESOLVERS_PATH),
        os.path.join(SCHEMAS_PATH),
        os.path.join(SCHEMAS_PATH, app),
        os.path.join(GRPC_PATH, 'Proto'),
        os.path.join(GRPC_PATH, 'Generated'),
        os.path.join(GRPC_PATH, 'Server'),
        os.path.join(GRPC_PATH, 'Client'),
        os.path.join(INFRA_PATH, 'Repositories', app)
    ]
    
    for directory in directories:
        os.makedirs(directory, exist_ok=True)

    # Contenidos CRUD
    use_case_interface_name = f'{app}UseCaseInterface'
    use_case_interface_content = f'''
from abc import ABC, abstractmethod
from typing import Dict
from Osdental.Models.Token import AuthToken

class {use_case_interface_name}(ABC):

    @abstractmethod
    async def get_all_{name_method}(self, {token}, {data}) -> str: ...

    @abstractmethod
    async def get_{name_method}_by_id(self, {token}, {data}) -> str: ...

    @abstractmethod
    async def create_{name_method}(self, {token}, {data}) -> str: ...

    @abstractmethod
    async def update_{name_method}(self, {token}, {data}) -> str: ...

    @abstractmethod
    async def delete_{name_method}(self, {token}, {data}) -> str: ...
    '''


    use_case_content = f'''
from typing import Dict
from Osdental.Decorators.DecryptedData import process_encrypted_data
from Osdental.Models.Token import AuthToken
from ..Interfaces.{use_case_interface_name} import {use_case_interface_name}
from src.Domain.Interfaces.{app}RepositoryInterface import {app}RepositoryInterface

class {app}UseCase({use_case_interface_name}):

    def __init__(self, repository: {app}RepositoryInterface):
        self.repository = repository
        
    @process_encrypted_data()
    async def get_all_{name_method}(self, {token}, {data}) -> str: pass

    @process_encrypted_data()        
    async def get_{name_method}_by_id(self, {token}, {data}) -> str: pass
        
    @process_encrypted_data()    
    async def create_{name_method}(self, {token}, {data}) -> str: pass

    @process_encrypted_data()
    async def update_{name_method}(self, {token}, {data}) -> str: pass

    @process_encrypted_data()
    async def delete_{name_method}(self, {token}, {data}) -> str: pass
    '''


    repository_interface_name = f'{app}RepositoryInterface'
    repository_interface_content = f'''
from abc import ABC, abstractmethod
from sqlalchemy import text, RowMapping
from typing import List

class {repository_interface_name}(ABC):

    @abstractmethod
    async def get_all_{name_method}(self, {data}) -> List[RowMapping]: ...

    @abstractmethod
    async def get_{name_method}_by_id(self, id: str) -> RowMapping: ...

    @abstractmethod
    async def create_{name_method}(self, {data}) -> str: ...

    @abstractmethod
    async def update_{name_method}(self, id: str, {data}) -> str: ...
    
    @abstractmethod
    async def delete_{name_method}(self, id: str) -> str: ...
    '''


    repository_content = f'''
from typing import List
from sqlalchemy import text, RowMapping
from sqlalchemy.ext.asyncio import AsyncSession
from src.Domain.Interfaces.{repository_interface_name} import {repository_interface_name}

class {app}Repository({repository_interface_name}):
    
    def __init__(self, session: AsyncSession):
        self.session = session    

    async def get_all_{name_method}(self, {data}) -> List[RowMapping]: pass
    
    async def get_{name_method}_by_id(self, id: str) -> RowMapping: pass

    async def create_{name_method}(self, {data}) -> str: pass
            
    async def update_{name_method}(self, id: str, {data}) -> str: pass
        
    async def delete_{name_method}(self, id: str) -> str: pass
    '''


    query_graphql = f'''type Query {{
    getAll{app}(data: String!): {api_type_response}
    get{app}ById(data: String!): {api_type_response}
}}
    '''

    mutation_graphql = f'''type Mutation {{
    create{app}(data: String!): {api_type_response}
    update{app}(data: String!): {api_type_response}
    delete{app}(data: String!): {api_type_response}
}}
    '''

    resolver_content_init = f"""
from .{app}Resolver import {app}Resolver

{name_method}_query_resolvers = {{
    'getAll{app}': {app}Resolver.resolve_get_all_{name_method},
    'get{app}ById': {app}Resolver.resolve_get_{name_method}_by_id
}}

{name_method}_mutation_resolvers = {{
    'create{app}': {app}Resolver.resolve_create_{name_method},
    'update{app}': {app}Resolver.resolve_update_{name_method},
    'delete{app}': {app}Resolver.resolve_delete_{name_method}
}}
    """

    resolver_content = f'''
from Osdental.Decorators.AuditLog import handle_audit_and_exception
from src.Application.UseCases.{app}UseCase import {app}UseCase
from src.Infrastructure.Repositories.{app}.{app}Repository import {app}Repository

use_case = {app}UseCase({app}Repository)

class {app}Resolver:
        
    @staticmethod
    @handle_audit_and_exception()
    async def resolve_get_all_{name_method}(_, info, data):
        return await use_case.get_all_{name_method}(info=info, aes_data=data)

    @staticmethod
    @handle_audit_and_exception()
    async def resolve_get_{name_method}_by_id(_, info, data):
        return await use_case.get_{name_method}_by_id(info=info, aes_data=data)

    @staticmethod
    @handle_audit_and_exception()
    async def resolve_create_{name_method}(_, info, data):
        return await use_case.create_{name_method}(info=info, aes_data=data)

    @staticmethod
    @handle_audit_and_exception()
    async def resolve_update_{name_method}(_, info, data):
        return await use_case.update_{name_method}(info=info, aes_data=data)
    
    @staticmethod
    @handle_audit_and_exception()
    async def resolve_delete_{name_method}(_, info, data):
        return await use_case.delete_{name_method}(info=info, aes_data=data)
    '''

    graphql_content_init = f'''
from ariadne import gql
from ariadne import QueryType, MutationType
from ariadne import make_executable_schema
from pathlib import Path
from ..Graphql.Resolvers.{app} import {name_method}_query_resolvers, {name_method}_mutation_resolvers

def load_schemas():
    schema_dir = Path(__file__).parent / 'Schemas'
    schemas = [schema.read_text() for schema in schema_dir.rglob('*.graphql')]
    return gql('\\n'.join(schemas))

type_defs = load_schemas()

query_resolvers = {{
    **{name_method}_query_resolvers,
}}

mutation_resolvers = {{
    **{name_method}_mutation_resolvers,
}}

query = QueryType()
mutation = MutationType()

for field, resolver in query_resolvers.items():
    query.set_field(field, resolver)

for field, resolver in mutation_resolvers.items():
    mutation.set_field(field, resolver)

# Executable Schema
schema = make_executable_schema(type_defs, query, mutation)
    '''


    response_content = '''
type Response {
    status: String
    message: String
    data: String
}
    '''

    init_file = '__init__.py'
    
    repository_init_content = f"""
from enum import StrEnum 

class {app}Query(StrEnum):
    GET_ALL_{app_upper} = ''' EXEC '''

    GET_{app_upper}_BY_ID = ''' EXEC '''

    CREATE_{app_upper} = ''' EXEC ''' 

    UPDATE_{app_upper} = ''' EXEC ''' 

    DELETE_{app_upper} = ''' EXEC ''' 
    """

    proto_app_content = f'''
syntax = "proto3";

package {app.lower()};

import "Common.proto";

// gRPC Service
    '''

    common_proto = '''
syntax = "proto3";

package common;

message Request { bytes data = 1; }
message Response { string status=1; string message=2; bytes data=3; }https://chatgpt.com/c/68a35293-1a34-832b-9bec-09e0513a52e3
message Empty {}
'''
    server = '''
from grpc.experimental import aio


async def serve():
    server = aio.server()

    # Register each service
    # Code Here ...

    # Single port exposed
    server.add_insecure_port("[::]:50051")

    await server.start()
    await server.wait_for_termination()
'''

    grpc_client = f'''
import grpc
from Osdental.Models.Response import Response

class {app}Client:

    def __init__(self, host="localhost", port=50051):
        # Connecting to the gRPC server
        self.channel = grpc.aio.insecure_channel(f"{{host}}:{{port}}")
        self.stub = 'Connects your client to the gRPC server via the channel and gives you access to all the service methods.'

    # Implement your RPC methods to consume

'''
    files = {
        os.path.join(APP_PATH, 'UseCases', f'{app}UseCase.py'): use_case_content,
        os.path.join(APP_PATH, 'Interfaces', f'{app}UseCaseInterface.py'): use_case_interface_content,
        os.path.join(DOMAIN_PATH, 'Interfaces', f'{app}RepositoryInterface.py'): repository_interface_content,
        os.path.join(RESOLVERS_PATH, app, init_file): resolver_content_init,
        os.path.join(RESOLVERS_PATH, app, f'{app}Resolver.py'): resolver_content,
        os.path.join(GRAPHQL_PATH, init_file): graphql_content_init, 
        os.path.join(SCHEMAS_PATH, app, 'Query.graphql'): query_graphql,
        os.path.join(SCHEMAS_PATH, app, 'Mutation.graphql'): mutation_graphql,
        os.path.join(SCHEMAS_PATH, 'Response.graphql'): response_content,
        os.path.join(INFRA_PATH, 'Repositories', app, init_file): repository_init_content,
        os.path.join(INFRA_PATH, 'Repositories', app, f'{app}Repository.py'): repository_content,
        os.path.join(GRPC_PATH, 'Proto', 'Common.proto'): common_proto,
        os.path.join(GRPC_PATH, 'Proto', f'{app}Service.proto'): proto_app_content,
        os.path.join(GRPC_PATH, 'Server', 'Server.py'): server,
        os.path.join(GRPC_PATH, 'Client', f'{app}Client.py'): grpc_client
    }
    for file_path in files:
        os.makedirs(os.path.dirname(file_path), exist_ok=True)

    for file_path, content in files.items():
        if not os.path.exists(file_path):
            with open(file_path, 'w') as f:
                f.write(content)

    logger.info(Message.HEXAGONAL_SERVICE_CREATED_MSG)

@cli.command()
@click.argument('port')
def start(port: int):
    """Start the FastAPI server.."""
    try:
        subprocess.run(['uvicorn', 'app:app', '--port', str(port), '--reload'], check=True)
    except subprocess.CalledProcessError as e:
        logger.error(f'{Message.SERVER_NETWORK_ACCESS_ERROR_MSG}: {e}')


@cli.command()
@click.argument('port')
def serve(port: int):
    """Set up the FastAPI server accessible from any machine."""
    try:
        subprocess.run(['uvicorn', 'app:app', '--host', '0.0.0.0', '--port', str(port), '--reload'], check=True)
    except subprocess.CalledProcessError as e:
        logger.error(f'{Message.SERVER_NETWORK_ACCESS_ERROR_MSG}: {e}')


@cli.command()
@click.argument('redis_env')
async def clean_redis(redis_env: str):
    try:
        from Osdental.RedisCache.Redis import RedisCacheAsync
        redis_url = os.getenv(redis_env)
        if not redis_url:
            logger.warning(f'Environment variable not found: {redis_env}')
            return
        
        redis = RedisCacheAsync(redis_url=redis_url)
        await redis.flush()
        logger.info(Message.REDIS_CLEANUP_SUCCESS_MSG)
    except Exception as e:
        logger.error(f'{Message.REDIS_CLEANUP_ERROR_MSG}: {e}')


@cli.command()
@click.argument('name')
def proto_files(name: str):
    proto_dir = r'.\src\Infrastructure\Grpc\Proto'
    gen_dir = r'.\src\Infrastructure\Grpc\Generated'

    if platform.system() == 'Windows':
        # Ejecuta el comando de grpc_tools.protoc
        cmd = f'''python -m grpc_tools.protoc -I={proto_dir} \
        --python_out={gen_dir} \
        --grpc_python_out={gen_dir} \
        {proto_dir}\\{name}Service.proto \
        {proto_dir}\\Common.proto'''
    else:
        # Linux / macOS
        cmd = f'''python -m grpc_tools.protoc -I={proto_dir} \
        --python_out={gen_dir} \
        --grpc_python_out={gen_dir} \
        {proto_dir}/{name}Service.proto \
        {proto_dir}/Common.proto'''

    subprocess.run(cmd, shell=True)
    logger.info(Message.PROTO_FILES_GENERATED_MSG)


@click.command()
@click.option("--host", default="0.0.0.0", help='Host where to set up the server')
@click.option("--port", default=5000, help='Server port')
@click.option("--workers", default=4, help='Number of workers')
def run_server(host, port, workers):
    """Launch the application with uvicorn on Windows or gunicorn on other systems"""
    if sys.platform.startswith("win"):
        cmd = [
            sys.executable, "-m", "uvicorn",
            "app:app",
            "--host", host,
            "--port", str(port),
            "--workers", str(workers)
        ]
    else:
        cmd = [
            sys.executable, "-m", "gunicorn",
            "app:app",
            "-k", "uvicorn.workers.UvicornWorker",
            "--bind", f"{host}:{port}",
            "--workers", str(workers)
        ]
    
    logger.info(f"Running: {' '.join(cmd)}")
    subprocess.run(cmd)
    
if __name__ == "__main__":
    cli()



from enum import StrEnum
class Profile(StrEnum):
    SUPER_ADMIN = 'SPAU'
    ADMIN_OSD = 'OSDA'
    MARKETING = 'OSDMK'
    EXT_ADMIN = 'ADEX'
    EXT_USER = 'EXTU'
    BOOKKEEPER = 'BKEU'
    PRODENT = 'OSDBI'
    LIMITED = 'LTD'


import re

class TextProcessor:

    @staticmethod
    def to_lower(text: str) -> str:
        """Converts the text to lowercase."""
        return text.lower()

    @staticmethod
    def to_upper(text: str) -> str:
        """Converts the text to uppercase."""
        return text.upper()

    @staticmethod
    def capitalize(text: str) -> str:
        """Capitalizes the first character of the text."""
        return text.capitalize()

    @staticmethod
    def title_case(text: str) -> str:
        """Capitalizes the first letter of each word in the text."""
        return text.title()
    
    @staticmethod
    def is_numeric(text: str) -> bool:
        """Checks if the text consists only of digits."""
        return text.isdigit()

    @staticmethod
    def is_alpha(text: str) -> bool:
        """Checks if the text consists only of alphabetic characters."""
        return text.isalpha()

    @staticmethod
    def remove_whitespace(text: str) -> str:
        """Removes extra whitespace from the text."""
        return ' '.join(text.split())

    @staticmethod
    def remove_special_chars(text: str, allowed: str = '') -> str:
        """Removes all special characters from the text, except those allowed."""
        return re.sub(fr'[^\w\s{re.escape(allowed)}]', '', text)
    
    @staticmethod
    def concatenate(*parts: str, sep: str = ' ') -> str:
        """Concatenates multiple string parts using the given separator."""
        return sep.join(parts)
    
    @staticmethod
    def contains(text: str, sub: str, case_sensitive: bool = True) -> bool:
        """Checks if the text contains the specified substring."""
        return sub in text if case_sensitive else sub.lower() in text.lower()
    
    @staticmethod
    def clean_word(text: str) -> str:
        """Cleans the text by removing special characters and converting to lowercase."""
        text = text.lower().strip()
        return re.sub(r'[^a-z0-9\s]', '', text)

    @staticmethod
    def clean_slug(text: str, sep: str = '_') -> str:
        """Cleans the text and converts it into a slug, using the given separator."""
        text = TextProcessor.clean_word(text)
        return sep.join(text.split())

    @staticmethod
    def remove_prefix(text: str, prefix: str) -> str:
        """Removes the specified prefix from the text if present."""
        return text[len(prefix):] if text.startswith(prefix) else text

    @staticmethod
    def remove_suffix(text: str, suffix: str) -> str:
        """Removes the specified suffix from the text if present."""
        return text[:-len(suffix)] if text.endswith(suffix) else text

    @staticmethod
    def truncate(text: str, length: int, ellipsis: bool = False) -> str:
        """Truncates the text to a specific length, optionally adding ellipsis."""
        return text[:length] + ('...' if ellipsis and len(text) > length else "")

    @staticmethod
    def slugify(text: str) -> str:
        """
        Converts the text to a URL-friendly slug format:
        - Normalizes
        - Removes non-word characters
        - Converts to lowercase
        - Replaces spaces and hyphens with single hyphen
        """
        text = TextProcessor.normalize(text)
        text = re.sub(r"[^\w\s-]", "", text).strip().lower()
        return re.sub(r"[-\s]+", "-", text)

    @staticmethod
    def normalize(text: str) -> str:
        """Placeholder for text normalization (e.g., unicode normalization)."""
        return re.sub(r'\s+', ' ', text.strip())

    



import re
import json
from datetime import datetime
from fastapi import Request
from tzlocal import get_localzone
from Osdental.ServicesBus.TaskQueue import task_queue
from Osdental.ExternalHttp.Client import APIClient
from Osdental.Models.Token import AuthToken
from Osdental.Encryptor.Jwt import JWT
from Osdental.Handlers.Instances import environment, microservice_name, microservice_version, aes, jwt_user_key
from Osdental.Handlers.DBSecurityQuery import DBSecurityQuery
from Osdental.Shared.Enums.Constant import Constant
from Osdental.Shared.Logger import logger
class CustomRequest:

    def __init__(self, request: Request):
        self.request = request
        self.local_tz = get_localzone()
        self.client = APIClient()

    async def send_to_service_bus(self) -> None:
        legacy = await DBSecurityQuery.get_legacy_data()
        message_in = await self.request.json()  
        request_data = Constant.DEFAULT_EMPTY_VALUE  
        match = re.search(r'data:\s*"([^"]+)"', message_in.get('query', ''))
        if match:
            encrypted_data = match.group(1)
            request_data = aes.decrypt(legacy.aes_key_user, encrypted_data)

        x_forwarded_for = self.request.headers.get('X-Forwarded-For')
        if x_forwarded_for:
            user_ip = x_forwarded_for.split(',')[0]
        else:
            user_ip = self.request.client.host

        if user_ip:
            location = await self.__get_location(user_ip)

        message_json = {
            'idMessageLog': self.request.headers.get('Idmessagelog'),
            'type': Constant.RESPONSE_TYPE_REQUEST,
            'environment': environment,
            'dateExecution': datetime.now(self.local_tz).strftime('%Y-%m-%d %H:%M:%S'),
            'header': json.dumps(dict(self.request.headers)),
            'microServiceUrl': str(self.request.url),
            'microServiceName': microservice_name,
            'microServiceVersion': microservice_version,
            'serviceName': message_in.get('operationName'),
            'machineNameUser': self.request.headers.get('Machinenameuser', Constant.DEFAULT_EMPTY_VALUE),
            'ipUser': user_ip or Constant.DEFAULT_EMPTY_VALUE,
            'userName': self.request.headers.get('Username', Constant.DEFAULT_EMPTY_VALUE),
            'localitation': location or Constant.DEFAULT_EMPTY_VALUE,
            'httpMethod': self.request.method,
            'httpResponseCode': Constant.DEFAULT_EMPTY_VALUE,
            'messageIn': request_data,
            'messageOut': Constant.DEFAULT_EMPTY_VALUE,
            'errorProducer': Constant.DEFAULT_EMPTY_VALUE,
            'auditLog': Constant.MESSAGE_LOG_INTERNAL,
            'batch': Constant.DEFAULT_EMPTY_VALUE,
            'idUser': self.__get_user_id(legacy.aes_key_user)
        }
        await task_queue.enqueue(message_json)

    async def __get_location(self, ip: str) -> str:
        try:
            response = await self.client.get(f'https://ipapi.co/{ip}/json/')
            if response.status_code == 200:
                data = response.json()
                return json.dumps({
                    'ip': ip,
                    'city': data.get('city'),
                    'region': data.get('region'),
                    'country': data.get('country_name'),
                    'latitude': data.get('latitude'),
                    'longitude': data.get('longitude'),
                })
            else:
                return Constant.DEFAULT_EMPTY_VALUE
        except Exception as e:
            logger.error(f'Error fetching location for IP {ip}: {e}')
            return Constant.DEFAULT_EMPTY_VALUE

    def __get_user_id(self, aes_key_user: str) -> str:
        authorization = self.request.headers.get('Authorization')
        user_token = None
        user_token_encrypted = None
        if authorization and authorization.startswith('Bearer '):
            user_token_encrypted = authorization.split(' ')[1]

        if user_token_encrypted:
            user_token = aes.decrypt(aes_key_user, user_token_encrypted)
            payload = JWT.extract_payload(user_token, jwt_user_key)
            token = AuthToken.from_jwt(payload)
            return token.id_user
        else:
            return Constant.DEFAULT_EMPTY_VALUE


import os
import asyncio
from typing import Dict
from dotenv import load_dotenv
from tenacity import retry, stop_after_attempt, wait_fixed
from Osdental.ServicesBus.ServicesBus import ServicesBus
from Osdental.Shared.Logger import logger

load_dotenv(dotenv_path='.env', override=True)

class TaskQueue:
    """Queue to manage tasks in order and asynchronously."""
    conn_str = os.getenv('CONNECTION_STRING')
    queue_name = os.getenv('QUEUE')
    
    def __init__(self):
        self.service_bus = ServicesBus(self.conn_str, self.queue_name)
        self.queue = asyncio.Queue()
        

    def start_processing(self):
        """Start processing tasks in the background."""
        try:
            loop = asyncio.get_running_loop()
        except RuntimeError:
            loop = asyncio.new_event_loop()
            asyncio.set_event_loop(loop)

        loop.create_task(self.process_tasks())

    @retry(stop=stop_after_attempt(3), wait=wait_fixed(5))
    async def enqueue(self, message:Dict[str,str]):
        """Add a task to the queue."""
        await self.queue.put(message)


    async def process_tasks(self):
        """Process tasks from the queue in order."""
        while True:
            message = await self.queue.get()
            try:
                await self.service_bus.send_message(message)
            except Exception as e:
                logger.error(f'Message queuing error: {str(e)}')
            finally:
                self.queue.task_done()


task_queue = TaskQueue()
task_queue.start_processing()


from azure.storage.blob.aio import BlobServiceClient
from Osdental.Handlers.DBCatalogQuery import DBCatalogQuery
from Osdental.Exception.ControlledException import AzureException
from Osdental.Shared.Logger import logger
from Osdental.Shared.Enums.Message import Message

class BlobStorage: 

    def __init__(self):
        self.catalog_handler = DBCatalogQuery()
    
    async def get_file(self, file_path:str) -> bytes:
        """ Download a file from blob storage """
        try:
            catalog = await self.catalog_handler.get_catalog_data('BlobStorage')
            blob_service_client = BlobServiceClient.from_connection_string(catalog.get('connectionString'))
            async with blob_service_client:
                container_client = blob_service_client.get_container_client(catalog.get('containerName'))
                blob_client = container_client.get_blob_client(file_path)
                blob_data = await blob_client.download_blob()
                file_bytes = await blob_data.readall()
                return file_bytes
        except Exception as e:
            logger.error(f'Unexpected blob storage error when retrieving file: {str(e)}')
            raise AzureException(message=Message.UNEXPECTED_ERROR_MSG, error=str(e)) 

    async def store_file(self, file_bytes:bytes, file_path:str) -> None:
        """ Upload a file to blob storage """
        try:
            catalog = await self.catalog_handler.get_catalog_data('BlobStorage')
            blob_service_client = BlobServiceClient.from_connection_string(catalog.get('connectionString'))
            async with blob_service_client:
                container_client = blob_service_client.get_container_client(catalog.get('containerName'))
                blob_client = container_client.get_blob_client(file_path)
                await blob_client.upload_blob(file_bytes, overwrite=True)
        except Exception as e:
            logger.error(f'Unexpected blob storage error when saving file: {str(e)}')
            raise AzureException(message=Message.UNEXPECTED_ERROR_MSG, error=str(e)) 

    async def delete_file(self, file_path:str) -> None:
        """ Delete a file from blob storage """
        try:
            catalog = await self.catalog_handler.get_catalog_data('BlobStorage')
            blob_service_client = BlobServiceClient.from_connection_string(catalog.get('connectionString'))
            async with blob_service_client:
                container_client = blob_service_client.get_container_client(catalog.get('containerName'))
                blob_client = container_client.get_blob_client(file_path)
                await blob_client.delete_blob()
        except Exception as e:
            logger.error(f'Unexpected blob storage error when deleting file: {str(e)}')
            raise AzureException(message=Message.UNEXPECTED_ERROR_MSG, error=str(e)) 



from dataclasses import dataclass, field, asdict
from enum import Enum
from Osdental.Shared.Enums.Code import Code
from Osdental.Shared.Enums.Message import Message

@dataclass
class Response:
    status: str = field(default=Code.PROCESS_SUCCESS_CODE)
    message: str = field(default=Message.PROCESS_SUCCESS_MSG)
    data: str = field(default=None)

    def __post_init__(self):
        # Asegura que status y message sean str si son Enum
        if isinstance(self.status, Enum):
            self.status = str(self.status)
        if isinstance(self.message, Enum):
            self.message = str(self.message)

    def send(self):
        return asdict(self)
